class BudgetsController < ApplicationController
  helper :queries
  include QueriesHelper
  include BudgetsHelper
  helper :budgets_queries
  include BudgetsQueriesHelper

  def index
    @project = Project.find_by_identifier(params["project_id"])
    use_session = !request.format.csv?
    retrieve_query(BudgetQuery, use_session)

    if @query.valid?
      respond_to do |format|
        format.html {
          @budget_count = @query.budget_count
          @budget_pages = Paginator.new @budget_count, per_page_option, params["page"]
          @budgets = @query.budgets(:offset => @budget_pages.offset, :limit => @budget_pages.per_page)
          render :layout => !request.xhr?
        }
        format.api {
          @offset, @limit = api_offset_and_limit
          @query.column_names = %w(author)
          @budget_count = @query.budget_count
          @budgets = @query.budgets(:offset => @offset, :limit => @limit)
          Issue.load_visible_relations(@budgets) if include_in_api_response?("relations")
        }
      end
    else
      respond_to do |format|
        format.html { render :layout => !request.xhr? }
        format.any(:atom, :csv, :pdf) { head 422 }
        format.api { render_validation_errors(@query) }
      end
    end
  rescue ActiveRecord::RecordNotFound
    render_404
  end

  def show
    @budget = Budget.find_by_id(params[:id])
    unless @budget && @budget.visible?
      render_404
      return
    end

    respond_to do |format|
      format.html { render :layout => !request.xhr? }
      format.api
    end
  end

  def new
    @project = Project.find_by_identifier(params["project_id"])
    @projects = Issue.allowed_target_projects(User.current, @project)
    @budget = Budget.new()
    @budget.safe_attributes = params[:budget]
    respond_to do |format|
      format.html { render :action => "new", :layout => !request.xhr? }
    end
  end

  def create
    @project = Project.find_by_identifier(params["project_id"])
    @projects = Issue.allowed_target_projects(User.current, @project)
    @budget = Budget.new()
    @budget.safe_attributes = params[:budget]
    @budget.created_by = User.current

    if @budget.save
      respond_to do |format|
        format.html {
          flash[:notice] = l(
            :notice_budget_successful_create,
            :id => view_context.link_to(
              @budget.budget_identifier,
              _project_budget_path(@project, @budget)
            ),
          )
          if params[:continue]
            redirect_to _new_project_budget_path(@project)
          else
            redirect_to _edit_project_budget_path(@project, @budget)
          end
        }
        format.api { render :action => "show", :status => :created, :location => budget_url(@budget) }
      end
    else
      respond_to do |format|
        format.html { render :action => "new" }
        format.api { render_validation_errors(@budget) }
      end
    end
  end

  def edit
    @budget = Budget.find_by_id(params[:id])
    @project = Project.find_by_identifier(params["project_id"])
    @projects = Issue.allowed_target_projects(User.current, @project)
  end

  def update
    @project = Project.find_by_identifier(params["project_id"])
    @projects = Issue.allowed_target_projects(User.current, @project)

    @budget = Budget.find_by_id(params[:id])
    @budget.safe_attributes = params[:budget]
    @budget.updated_by = User.current

    if @budget.save
      respond_to do |format|
        format.html {
          flash[:notice] = l(:notice_successful_update)
          redirect_to_referer_or edit_budget_path(@budget)
        }
        format.api { render_api_ok }
      end
    else
      @projects = Project.active.all

      respond_to do |format|
        format.html { render :action => :edit }
        format.api { render_validation_errors(@budget) }
      end
    end
  end

  def destroy
    @budget = Budget.find_by_id(params[:id])
    @budget.destroy
    respond_to do |format|
      format.html { redirect_back_or_default(budgets_path) }
      format.api { render_api_ok }
    end
  end
end
