class EntitiesController < ApplicationController
  layout "cost_admin"
  # self.main_menu = false

  helper :sort
  include SortHelper

  def index
    sort_init "name", "asc"
    sort_update %w(name)

    case params[:format]
    when "xml", "json"
      @offset, @limit = api_offset_and_limit
    else
      @limit = per_page_option
    end

    @name = params[:name] || ""

    scope = Entity.all
    scope = scope.like(params[:name]) if params[:name].present?

    @entity_count = scope.count
    @entity_pages = Paginator.new @entity_count, @limit, params["page"]
    @offset ||= @entity_pages.offset
    @entities = scope.order(sort_clause).limit(@limit).offset(@offset).to_a

    respond_to do |format|
      format.html {
        render :layout => !request.xhr?
      }
      format.csv {
        send_data(users_to_csv(scope.order(sort_clause)), :type => "text/csv; header=present", :filename => "users.csv")
      }
      format.api
    end
  end

  def show
    @entity = Entity.find_by_id(params[:id])
    unless @entity.visible?
      render_404
      return
    end

    respond_to do |format|
      format.html {
        render :layout => "cost_admin"
      }
      format.api
    end
  end

  def new
    @entity = Entity.new()
    @entity.safe_attributes = params[:entity]
  end

  def create
    @entity = Entity.new()
    @entity.safe_attributes = params[:entity]
    @entity.created_by = User.current

    if @entity.save
      respond_to do |format|
        format.html {
          flash[:notice] = l(:notice_entity_successful_create, :id => view_context.link_to(@entity.name, entity_path(@entity)))
          if params[:continue]
            redirect_to new_entity_path
          else
            redirect_to edit_entity_path(@entity)
          end
        }
        format.api { render :action => "show", :status => :created, :location => entity_url(@entity) }
      end
    else
      respond_to do |format|
        format.html { render :action => "new" }
        format.api { render_validation_errors(@entity) }
      end
    end
  end

  def edit
    @entity = Entity.find_by_id(params[:id])
  end

  def update
    @entity = Entity.find_by_id(params[:id])
    @entity.safe_attributes = params[:entity]
    @entity.updated_by = User.current

    if @entity.save
      respond_to do |format|
        format.html {
          flash[:notice] = l(:notice_successful_update)
          redirect_to_referer_or edit_entity_path(@entity)
        }
        format.api { render_api_ok }
      end
    else
      respond_to do |format|
        format.html { render :action => :edit }
        format.api { render_validation_errors(@entity) }
      end
    end
  end

  def destroy
    @entity = Entity.find_by_id(params[:id])
    @entity.destroy
    respond_to do |format|
      format.html { redirect_back_or_default(entities_path) }
      format.api { render_api_ok }
    end
  end
end
