class ExpensesController < ApplicationController
  helper :queries
  include QueriesHelper
  include ExpensesHelper
  helper :expenses_queries
  include ExpensesQueriesHelper

  def index
    @project = Project.find_by_identifier(params["project_id"])
    use_session = !request.format.csv?
    retrieve_query(ExpenseQuery, use_session)

    if @query.valid?
      respond_to do |format|
        format.html {
          @expense_count = @query.expense_count
          @expense_pages = Paginator.new @expense_count, per_page_option, params["page"]
          @expenses = @query.expenses(:offset => @expense_pages.offset, :limit => @expense_pages.per_page)
          render :layout => !request.xhr?
        }
        format.api {
          @offset, @limit = api_offset_and_limit
          @query.column_names = %w(author)
          @expense_count = @query.expense_count
          @expenses = @query.expenses(:offset => @offset, :limit => @limit)
          Issue.load_visible_relations(@expenses) if include_in_api_response?("relations")
        }
      end
    else
      respond_to do |format|
        format.html { render :layout => !request.xhr? }
        format.any(:atom, :csv, :pdf) { head 422 }
        format.api { render_validation_errors(@query) }
      end
    end
  rescue ActiveRecord::RecordNotFound
    render_404
  end

  def show
    @expense = Expense.find_by_id(params[:id])
    unless @expense && @expense.visible?
      render_404
      return
    end

    respond_to do |format|
      format.html { render :layout => !request.xhr? }
      format.api
    end
  end

  def new
    @project = Project.find_by_identifier(params["project_id"])
    @projects = Issue.allowed_target_projects(User.current, @project)
    @expense = Expense.new()
    @expense.safe_attributes = params[:expense]
    respond_to do |format|
      format.html { render :action => "new", :layout => !request.xhr? }
    end
  end

  def create
    @project = Project.find_by_identifier(params["project_id"])
    @projects = Issue.allowed_target_projects(User.current, @project)
    @expense = Expense.new()
    @expense.safe_attributes = params[:expense]
    @expense.created_by = User.current

    if @expense.save
      respond_to do |format|
        format.html {
          flash[:notice] = l(
            :notice_expense_successful_create,
            :id => view_context.link_to(
              @expense.expense_identifier,
              _project_expense_path(@project, @expense)
            ),
          )
          if params[:continue]
            redirect_to _new_project_expense_path(@project)
          else
            redirect_to _edit_project_expense_path(@project, @expense)
          end
        }
        format.api { render :action => "show", :status => :created, :location => expense_url(@expense) }
      end
    else
      respond_to do |format|
        format.html { render :action => "new" }
        format.api { render_validation_errors(@expense) }
      end
    end
  end

  def edit
    @expense = Expense.find_by_id(params[:id])
    @project = Project.find_by_identifier(params["project_id"])
    @projects = Issue.allowed_target_projects(User.current, @project)
  end

  def update
    @project = Project.find_by_identifier(params["project_id"])
    @projects = Issue.allowed_target_projects(User.current, @project)

    @expense = Expense.find_by_id(params[:id])
    @expense.safe_attributes = params[:expense]
    @expense.updated_by = User.current

    if @expense.save
      respond_to do |format|
        format.html {
          flash[:notice] = l(:notice_successful_update)
          redirect_to_referer_or edit_expense_path(@expense)
        }
        format.api { render_api_ok }
      end
    else
      @projects = Project.active.all

      respond_to do |format|
        format.html { render :action => :edit }
        format.api { render_validation_errors(@expense) }
      end
    end
  end

  def destroy
    @expense = Expense.find_by_id(params[:id])
    @expense.destroy
    respond_to do |format|
      format.html { redirect_back_or_default(expenses_path) }
      format.api { render_api_ok }
    end
  end
end
