class Entity < ActiveRecord::Base
  include Redmine::SafeAttributes

  has_many :contract
  belongs_to :created_by, :class_name => "User"
  belongs_to :updated_by, :class_name => "User"

  validates_uniqueness_of :name, :if => Proc.new { |entity| entity.name_changed? && entity.name.present? }, :case_sensitive => false

  scope :sorted, lambda { order(*Contract.fields_for_order_statement) }

  scope :like, lambda { |q|
    q = q.to_s
    if q.blank?
      where({})
    else
      pattern = "%#{q}%"
      sql = +"LOWER(#{table_name}.name) LIKE LOWER(:p)"
      params = { :p => pattern }

      where(sql, params)
    end
  }

  safe_attributes(
    "name",
    "short_name"
  )

  def visible?(user = User.current)
    user.allowed_to?(:view_entities, nil) || user.admin?
  end

  def editable?(user = User.current)
    user.allowed_to?(:edit_entities, nil) || user.admin?
  end

  def deletable?
    user.allowed_to?(:edit_entities, nil) || user.admin?
  end

  def self.fields_for_order_statement(table = nil)
    table ||= table_name
    columns = ["name"]
    columns.uniq.map { |field| "#{table}.#{field}" }
  end

  def to_s
    name
  end

  def self.detect_by_keyword(entities, keyword)
    keyword = keyword.to_s
    return nil if keyword.blank?

    entity = nil
    entity ||= entities.detect { |a| keyword.casecmp(a.name.to_s) == 0 }
    entity ||= entities.detect { |a| keyword.casecmp(a.short_name.to_s) == 0 }

    entity
  end
end
