ActiveSupport::Reloader.to_prepare do
  unless Issue.included_modules.include?(RedmineCostAdmin::IssuePatch)
    Issue.send(:include, RedmineCostAdmin::IssuePatch)
  end
end

require_dependency "issue"

module RedmineCostAdmin
  module IssuePatch
    def self.included(base) # :nodoc:
      base.send(:include, InstanceMethods)
      base.include(ClassMethods)

      base.class_eval do
        unloadable
      end
    end

    module InstanceMethods
      def estimated_cost
        contract = assigned_to.valid_contract(project, start_date ? start_date : Time.now) if assigned_to

        return 0 unless contract && estimated_hours

        unit_price = case contract.reward_unit
          when "hour"
            contract.unit_price
          when "day"
            contract.unit_price / 8
          when "week"
            contract.unit_price / (8 * 5)
          when "month"
            contract.unit_price / (8 * 20)
          when "year"
            contract.unit_price / (8 * 244)
          else
            0
          end
        (estimated_hours * unit_price).round
      end

      def total_estimated_cost
        if leaf?
          estimated_cost
        else
          @total_estimated_cost ||= 0
          issues = self_and_descendants.visible
          issues.each do |issue|
            @total_estimated_cost += issue.estimated_cost
          end
          @total_estimated_cost
        end
      end

      def spent_cost
        spent_cost = 0
        time_entries.each do |te|
          contract = te.user.valid_contract(te.project, te.spent_on)
          if contract
            unit_price = case contract.reward_unit
              when "hour"
                contract.unit_price
              when "day"
                contract.unit_price / 8
              when "week"
                contract.unit_price / (8 * 5)
              when "month"
                contract.unit_price / (8 * 20)
              when "year"
                contract.unit_price / (8 * 244)
              else
                0
              end
          else
            unit_price = 0
          end
          spent_cost += (te.hours * unit_price)
        end
        spent_cost.round
      end

      def total_spent_cost
        if leaf?
          spent_cost
        else
          @total_spent_cost ||= 0
          issues = self_and_descendants.visible
          issues.each do |issue|
            @total_spent_cost += issue.spent_cost
          end
          @total_spent_cost
        end
      end
    end

    module ClassMethods
    end
  end
end
