ActiveSupport::Reloader.to_prepare do
  unless Project.included_modules.include?(RedmineCostAdmin::ProjectPatch)
    Project.send(:include, RedmineCostAdmin::ProjectPatch)
  end
end

require_dependency "project"

module RedmineCostAdmin
  module ProjectPatch
    def self.included(base) # :nodoc:
      base.send(:include, InstanceMethods)
      base.include(ClassMethods)

      base.class_eval do
        unloadable
        has_many :budgets, dependent: :destroy
        has_many :expenses, dependent: :destroy
        has_many :contracts, dependent: :destroy
      end
    end

    module InstanceMethods
      def total_estimated_cost
        cond = project_condition(Setting.display_subprojects_issues?)
        issues = Issue.visible.where(cond)
        total_estimated_cost = 0
        issues.each do |issue|
          total_estimated_cost += issue.estimated_cost
        end
        total_estimated_cost
      end

      def total_spent_cost
        cond = project_condition(Setting.display_subprojects_issues?)
        issues = Issue.visible.where(cond).preload(:assigned_to)
        total_spent_cost = 0
        issues.each do |issue|
          total_spent_cost += issue.spent_cost
        end
        total_spent_cost
      end
    end

    module ClassMethods
    end
  end
end
