ActiveSupport::Reloader.to_prepare do
  unless User.included_modules.include?(RedmineCostAdmin::UserPatch)
    User.send(:include, RedmineCostAdmin::UserPatch)
  end
end

require_dependency "user"

module RedmineCostAdmin
  module UserPatch
    def self.included(base) # :nodoc:
      base.send(:include, InstanceMethods)
      base.include(ClassMethods)

      base.class_eval do
        unloadable
        has_many :contracts, dependent: :destroy
      end
    end

    module InstanceMethods
      def valid_contract(project = nil, time = Time.now)
        contracts = Contract.where("contract_type != ? and user_id = ? and date_valid_from <= ? and date_valid_to >= ?", "deliverable", id, time, time)
        if contracts.any?
          if contracts.count == 1
            return contracts.first
          else
            # possible quasi delegation. need a project specified.
            return nil unless project
            return contracts.select { |c| c.project_id == project.id }
          end
        else
          nil
        end
      end

      def all_contracts
        Contract.where(:user_id => id).order(:contract_type => "ASC", :date_valid_to => "ASC")
      end
    end

    module ClassMethods
    end
  end
end
